@@ -1,8 +1,8 @@
-autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No monitored variables supplied"), data=NA, n.chains=2, inits = replicate(n.chains, NA), startburnin = 5000, startsample = 10000, psrf.target = 1.05, normalise.mcmc = TRUE, check.stochastic = TRUE, raftery.options = list(), crash.retry=1, plots = TRUE, thin.sample = TRUE, jags = findjags(), silent.jags = FALSE, interactive=TRUE, max.time=Inf, adaptive=list(type="burnin", length=200), modules=c(""), thin = 1, monitor.deviance = FALSE, monitor.pd = FALSE, monitor.popt = FALSE, keep.jags.files=FALSE, tempdir=TRUE, method=if(.Platform$OS.type=='unix' & .Platform$GUI!="AQUA") 'interruptible' else 'simple'){
+autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No monitored variables supplied"), data=NA, n.chains=2, inits = replicate(n.chains, NA), startburnin = 5000, startsample = 10000, psrf.target = 1.05, normalise.mcmc = TRUE, check.stochastic = TRUE, raftery.options = list(), crash.retry=1, plots = TRUE, thin.sample = TRUE, jags = findjags(), silent.jags = FALSE, interactive=TRUE, max.time=Inf, adaptive=list(type="burnin", length=200), modules=c(""), factories=c(""), thin = 1, monitor.deviance = FALSE, monitor.pd = FALSE, monitor.pd.i = FALSE, monitor.popt = FALSE, keep.jags.files=FALSE, tempdir=TRUE, method=if(.Platform$OS.type=='unix' & .Platform$GUI!="AQUA") 'interruptible' else 'simple'){
 
-	if(any(c(monitor.deviance, monitor.pd, monitor.popt))) modules <- c(modules, "dic")
+	if(any(c(monitor.deviance, monitor.pd, monitor.popt, monitor.pd.i))) modules <- c(modules, "dic")
 	modules <- unique(modules)
-	modules <- na.omit(modules[modules!=""])
+	modules <- as.character(na.omit(modules[modules!=""]))
 	
 	if(as.integer(thin)!=thin | thin < 1) stop("The value supplied for thin must be a positive integer")
 	
@@ -89,7 +89,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 	pre.time <- Sys.time()
 	
 	
-	pilot <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits = inits, burnin=startburnin, sample=startsample, adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE, jags = jags, psrf.target = psrf.target, normalise.mcmc=normalise.mcmc, check.stochastic = check.stochastic, modules=c(""), thin=thin, monitor.deviance=monitor.deviance, monitor.pd=monitor.pd, monitor.popt=monitor.popt, keep.jags.files=keep.jags.files, tempdir=tempdir, method=method)
+	pilot <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits = inits, burnin=startburnin, sample=startsample, adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE, jags = jags, psrf.target = psrf.target, normalise.mcmc=normalise.mcmc, check.stochastic = check.stochastic, modules=modules, factories=factories, thin=thin, monitor.deviance=monitor.deviance, monitor.pd=monitor.pd, monitor.pd.i=monitor.pd.i, monitor.popt=monitor.popt, keep.jags.files=keep.jags.files, tempdir=tempdir, method=method)
 	if(any(pilot=="Unable to load coda files")){
 		cat("An error occured during the simulation\n\n")
 		return(c("Error", "An error occured during the simulation"))
@@ -114,7 +114,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 			cat("\nThe simulation crashed; retrying...",newlines,sep="")			
 			crash.retry <- crash.retry - 1
 			oldpilot <- pilot
-			pilot <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits = inits, burnin=startburnin, sample=startsample, adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE, jags = jags, psrf.target = psrf.target, normalise.mcmc=normalise.mcmc, check.stochastic = check.stochastic, modules=c(""), thin=thin, monitor.deviance=monitor.deviance, monitor.pd=monitor.pd, monitor.popt=monitor.popt, keep.jags.files=keep.jags.files, tempdir=tempdir, method=method)
+			pilot <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits = inits, burnin=startburnin, sample=startsample, adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE, jags = jags, psrf.target = psrf.target, normalise.mcmc=normalise.mcmc, check.stochastic = check.stochastic, modules=modules, factories=factories, thin=thin, monitor.deviance=monitor.deviance, monitor.pd=monitor.pd, monitor.pd.i=monitor.pd.i, monitor.popt=monitor.popt, keep.jags.files=keep.jags.files, tempdir=tempdir, method=method)
 			
 			if(!any(names(pilot)=="crash.end")) break
 		}
@@ -126,25 +126,17 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 	otheroutputs <- vector("list")
 	if(monitor.deviance) otheroutputs <- c(otheroutputs, deviance=list(pilot$deviance))
 	if(monitor.pd) otheroutputs <- c(otheroutputs, pd=list(pilot$pd))
+	if(monitor.pd.i) otheroutputs <- c(otheroutputs, pd.i=list(pilot$pd.i))
 	if(monitor.popt) otheroutputs <- c(otheroutputs, popt=list(pilot$popt))
 	
 	final.mcmc <- pilot$mcmc
 	
-	cat("Calculating the Gelman-Rubin statistic....\n")
-	suppressWarnings(success <- try(convergence <- safe.gelman.diag(normalise.mcmc(final.mcmc, normalise=normalise.mcmc, warn=FALSE, check.stochastic = check.stochastic), transform=FALSE, autoburnin=TRUE), silent=TRUE))
+	cat("Calculating the Gelman-Rubin statistic for ", nvar(final.mcmc), " variables....\n", sep="")
+	suppressWarnings(success <- try(convergence <- safe.gelman.diag(normalise.mcmcfun(final.mcmc, normalise=normalise.mcmc, warn=FALSE, check.stochastic = check.stochastic), transform=FALSE, autoburnin=TRUE), silent=TRUE))
 	if(class(success)=="try-error"){
 		cat("An error occured while calculating the Gelman-Rubin statistic; aborting simulation.  Check that different chains have not been given the same starting values and random seeds.\n")
 		return(c("Error", "An error occured while calculating the Gelman-Rubin statistic"))
 	}
-	suppressWarnings(success <- try(autocorr <- autocorr.diag(normalise.mcmc(final.mcmc, normalise=FALSE, warn=FALSE, check.stochastic = check.stochastic)), silent=TRUE))
-	if(class(success)=="try-error"){
-		cat("An error occured while calculating the autocorrelation; aborting simulation\n")
-		return(c("Error", "An error occured while calculating the autocorrelation"))
-	}
-	
-	options(show.error.messages = FALSE)
-	suppressWarnings(fsummary <- summary(combine.mcmc(final.mcmc, collapse.chains=FALSE)))
-	options(show.error.messages = TRUE)
 	
 	convergence <- c(convergence, psrf.target=psrf.target)
 	class(convergence) <- "gelman.with.target"
@@ -177,7 +169,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 				mpsrfstring <- paste(" (multi-variate psrf = ", round(convergence$mpsrf, digits=3), ")", sep="")
 			}
 			
-			cat("The Gelman-Rubin statistic was above ", psrf.target, " for ", unconverged, " parameter(s) after ", n.iters, " iterations", mpsrfstring, ".  This may indicate poor convergence.\n", sep="")
+			cat("The Gelman-Rubin statistic was above ", psrf.target, " for ", unconverged, " parameter", if(unconverged>1) "s", " after ", n.iters, " iterations", mpsrfstring, ".  This may indicate poor convergence.\n", sep="")
 			updatesthrown <- updatesthrown + startsample
 			time.taken <- timestring(pre.time, Sys.time(), units="secs", show.units=FALSE)
 			stop <- time.taken > max.time
@@ -188,7 +180,21 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 				pre.time <- Sys.time()-time.taken
 			}
 			if(stop){
+				
+				cat("Calculating autocorrelation and summary statistics...\n")
+				
+				suppressWarnings(success <- try(autocorr <- safe.autocorr.diag(normalise.mcmcfun(final.mcmc, normalise=FALSE, warn=FALSE, check.stochastic = check.stochastic)), silent=TRUE))
+				if(class(success)=="try-error"){
+					cat("An error occured while calculating the autocorrelation; aborting simulation\n")
+					return(c("Error", "An error occured while calculating the autocorrelation"))
+				}
+
+				options(show.error.messages = FALSE)
+				suppressWarnings(fsummary <- summary(combine.mcmc(final.mcmc, collapse.chains=FALSE)))
+				options(show.error.messages = TRUE)
+				
 				cat("Returning UNCONVERGED simulation results\n\n")
+				
 				return(c(list(pilot.mcmc=final.mcmc, end.state=pilot$end.state, req.samples=NA, pilot.summary=fsummary, samples.to.conv=NA, psrf=convergence, autocorr=autocorr), otheroutputs))
 			}else{
 				finishconv <- FALSE
@@ -208,7 +214,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 				if(neededupdates > 0){
 					
 					oldadditional <- additional
-					additional <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits=pilot$end.state, burnin=burnadapt, sample=neededupdates, adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE, jags = jags, psrf.target = psrf.target, normalise.mcmc=normalise.mcmc, check.stochastic = check.stochastic, modules=c(""), thin=thin, monitor.deviance=monitor.deviance, monitor.pd=monitor.pd, monitor.popt=monitor.popt, keep.jags.files=keep.jags.files, tempdir=tempdir, method=method)
+					additional <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits=pilot$end.state, burnin=burnadapt, sample=neededupdates, adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE, jags = jags, psrf.target = psrf.target, normalise.mcmc=normalise.mcmc, check.stochastic = check.stochastic, modules=modules, factories=factories, thin=thin, monitor.deviance=monitor.deviance, monitor.pd=monitor.pd, monitor.pd.i=monitor.pd.i, monitor.popt=monitor.popt, keep.jags.files=keep.jags.files, tempdir=tempdir, method=method)
 					
 					if(any(pilot=="Unable to load coda files")){
 						cat("\nThere was an error in the second simulation, possibly due to bad initial values or Random Number Seed values obtained from the first simulation.  You could try using fewer chains.")
@@ -229,7 +235,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 							cat("\nThe simulation crashed; retrying...",newlines,sep="")
 							crash.retry <- crash.retry - 1
 							oldadd <- additional
-							additional <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits = oldadditional$end.state, burnin=burnadapt, sample=(neededupdates), adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE, jags = jags, psrf.target = psrf.target, normalise.mcmc=normalise.mcmc, check.stochastic = check.stochastic, modules=c(""), thin=thin, monitor.deviance=monitor.deviance, monitor.pd=monitor.pd, monitor.popt=monitor.popt, keep.jags.files=keep.jags.files, tempdir=tempdir, method=method)
+							additional <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits = oldadditional$end.state, burnin=burnadapt, sample=(neededupdates), adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE, jags = jags, psrf.target = psrf.target, normalise.mcmc=normalise.mcmc, check.stochastic = check.stochastic, modules=modules, factories=factories, thin=thin, monitor.deviance=monitor.deviance, monitor.pd=monitor.pd, monitor.pd.i=monitor.pd.i, monitor.popt=monitor.popt, keep.jags.files=keep.jags.files, tempdir=tempdir, method=method)
 							
 							if(!any(names(additional)=="crash.end")) break
 						}
@@ -239,22 +245,13 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 					
 					final.mcmc <- window(combine.mcmc(list(final.mcmc, additional$mcmc)), collapse.chains=FALSE,  start=(((niter(final.mcmc)+niter(additional$mcmc))-startsample)+1))
 					
-					cat("Calculating the Gelman-Rubin statistic....\n")
+					cat("Calculating the Gelman-Rubin statistic for ", nvar(final.mcmc), " variables....\n", sep="")
 					thrownaway <- thrownaway+neededupdates
-					suppressWarnings(success <- try(convergence <- safe.gelman.diag(normalise.mcmc(final.mcmc, normalise=normalise.mcmc, warn=FALSE, check.stochastic = check.stochastic), transform=FALSE, autoburnin=TRUE), silent=TRUE))
+					suppressWarnings(success <- try(convergence <- safe.gelman.diag(normalise.mcmcfun(final.mcmc, normalise=normalise.mcmc, warn=FALSE, check.stochastic = check.stochastic), transform=FALSE, autoburnin=TRUE), silent=TRUE))
 					if(class(success)=="try-error"){
 						cat("An error occured while calculating the Gelman-Rubin statistic; aborting simulation.  Check that different chains have not been given the same starting values and random seeds.\n")
 						return(c("Error", "An error occured while calculating the Gelman-Rubin statistic"))
 					}
-					suppressWarnings(success <- try(autocorr <- autocorr.diag(normalise.mcmc(final.mcmc, normalise=FALSE, warn=FALSE, check.stochastic = check.stochastic)), silent=TRUE))
-					if(class(success)=="try-error"){
-						cat("An error occured while calculating the autocorrelation; aborting simulation\n")
-						return(c("Error", "An error occured while calculating the autocorrelation"))
-					}
-					
-					options(show.error.messages = FALSE)
-					suppressWarnings(fsummary <- summary(combine.mcmc(final.mcmc, collapse.chains=FALSE)))
-					options(show.error.messages = TRUE)
 					
 					convergence <- c(convergence, psrf.target=psrf.target)
 					class(convergence) <- "gelman.with.target"
@@ -277,6 +274,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 					
 					if(monitor.deviance) otheroutputs$deviance <- additional$deviance
 					if(monitor.pd) otheroutputs$pd <- additional$pd
+					if(monitor.pd.i) otheroutputs$pd.i <- additional$pd.i
 					if(monitor.popt) otheroutputs$popt <- additional$popt
 					
 					if(class(convergence$mpsrf)!="numeric"){
@@ -286,7 +284,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 					}
 					
 					if(unconverged > 0){
-						cat("The Gelman-Rubin statistic was still above ", psrf.target, " for ", unconverged, " parameter(s) after ", updatesdone + neededupdates, " iterations", mpsrfstring, ".\n", sep="")
+						cat("The Gelman-Rubin statistic was still above ", psrf.target, " for ", unconverged, " parameter", if(unconverged>1) "s", " after ", updatesdone + neededupdates, " iterations", mpsrfstring, ".\n", sep="")
 						
 						stop <- time.taken > max.time
 						if(interactive){
@@ -295,7 +293,21 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 							pre.time <- Sys.time()-time.taken
 						}
 						if(stop){
+							
+							cat("Calculating autocorrelation and summary statistics...\n")
+							
+							suppressWarnings(success <- try(autocorr <-  safe.autocorr.diag(normalise.mcmcfun(final.mcmc, normalise=FALSE, warn=FALSE, check.stochastic = check.stochastic)), silent=TRUE))
+							if(class(success)=="try-error"){
+								cat("An error occured while calculating the autocorrelation; aborting simulation\n")
+								return(c("Error", "An error occured while calculating the autocorrelation"))
+							}
+
+							options(show.error.messages = FALSE)
+							suppressWarnings(fsummary <- summary(combine.mcmc(final.mcmc, collapse.chains=FALSE)))
+							options(show.error.messages = TRUE)
+							
 							cat("Returning UNCONVERGED simulation results\n\n")
+							
 							return(c(list(pilot.mcmc=final.mcmc, end.state=pilot$end.state, req.samples=NA, pilot.summary=fsummary, samples.to.conv=NA, psrf=convergence, autocorr=autocorr), otheroutputs))
 						}else{
 							cat("Extending the simulation to attempt to improve convergence...\n")
@@ -328,7 +340,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 	cat("\nCalculating the necessary sample length based on the Raftery and Lewis's diagnostic...\n")
 			
 	success <- try({
-	raftery.args$data <- normalise.mcmc(final.mcmc, normalise=FALSE, warn=FALSE, check.stochastic = check.stochastic)
+	raftery.args$data <- normalise.mcmcfun(final.mcmc, normalise=FALSE, warn=FALSE, check.stochastic = check.stochastic)
 	class(raftery.args) <- "list"
 	raftery <- do.call("raftery.diag", raftery.args)
 	})
@@ -354,7 +366,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 	dependancethreshold <- 3
 	
 	if(any(dependance > dependancethreshold) & killautocorr==FALSE){
-				cat("IMPORTANT:  The sample size(s) of monitored node(s) '", paste(dimnames(dependance)[[1]][apply(dependance, 1, function(x) if(any(x>dependancethreshold)) return(TRUE) else return(FALSE))], collapse="' & '"), "' have a high autocorrelation dependance in chain(s) ", paste(seq(1, n.chains)[apply(dependance, 2, function(x) if(any(x>dependancethreshold)) return(TRUE) else return(FALSE))], collapse= " & "), ".  Re-running the model with a different formulation or better initial values may help to reduce autocorrelation.\n", sep="")
+				cat("IMPORTANT:  The sample size of monitored node(s) '", paste(dimnames(dependance)[[1]][apply(dependance, 1, function(x) if(any(x>dependancethreshold)) return(TRUE) else return(FALSE))], collapse="' & '"), "' have a high autocorrelation dependance in chain(s) ", paste(seq(1, n.chains)[apply(dependance, 2, function(x) if(any(x>dependancethreshold)) return(TRUE) else return(FALSE))], collapse= " & "), ".  Re-running the model with a different formulation or better initial values may help to reduce autocorrelation.\n", sep="")
 	}
 	
 	#moreupdates <- ((max(sample+(max(0, burnin-startburnin)))-(startsample*n.chains))/n.chains)*1.05
@@ -383,6 +395,17 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 			cat("The model will need to be run for a further ", moreupdates, " updates.  This will take approximately ", timestring((firsttimetaken*moreupdates/(startsample+startburnin))), ".\n", sep="")
 			if(interactive & (timestring((time.taken*moreupdates/(startsample+startburnin)), units="s", show.units=FALSE)>60)) if(!ask("Continue with the simulation?")){
 				cat("Simulation aborted\n\n")
+				suppressWarnings(success <- try(autocorr <-  safe.autocorr.diag(normalise.mcmcfun(final.mcmc, normalise=FALSE, warn=FALSE, check.stochastic = check.stochastic)), silent=TRUE))
+				if(class(success)=="try-error"){
+					cat("An error occured while calculating the autocorrelation; aborting simulation\n")
+					return(c("Error", "An error occured while calculating the autocorrelation"))
+				}
+
+				options(show.error.messages = FALSE)
+				suppressWarnings(fsummary <- summary(combine.mcmc(final.mcmc, collapse.chains=FALSE)))
+				options(show.error.messages = TRUE)
+				
+				cat("Calculating autocorrelation and summary statistics...\n")
 				return(c(list(pilot.mcmc=final.mcmc, end.state=pilot$end.state, req.samples=max(sample), pilot.summary=fsummary, samples.to.conv=NA, psrf=convergence, autocorr=autocorr), otheroutputs))
 				
 			}
@@ -392,6 +415,16 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 			stop <- TRUE
 			if(interactive & (timestring((time.taken*moreupdates/(startsample+startburnin)), units="s", show.units=FALSE)>60)) stop <- !ask("Continue with the simulation?")
 			if(stop){
+				suppressWarnings(success <- try(autocorr <-  safe.autocorr.diag(normalise.mcmcfun(final.mcmc, normalise=FALSE, warn=FALSE, check.stochastic = check.stochastic)), silent=TRUE))
+				if(class(success)=="try-error"){
+					cat("An error occured while calculating the autocorrelation; aborting simulation\n")
+					return(c("Error", "An error occured while calculating the autocorrelation"))
+				}
+
+				options(show.error.messages = FALSE)
+				suppressWarnings(fsummary <- summary(combine.mcmc(final.mcmc, collapse.chains=FALSE)))
+				options(show.error.messages = TRUE)
+				
 				cat("Simulation aborted\n\n")
 				return(c(list(pilot.mcmc=final.mcmc, end.state=pilot$end.state, req.samples=max(sample), pilot.summary=fsummary, samples.to.conv=NA, psrf=convergence, autocorr=autocorr), otheroutputs))
 			}
@@ -402,6 +435,17 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 		if(interactive & (timestring((time.taken*moreupdates/(startsample+startburnin)), units="s", show.units=FALSE)>60)){
 			time.taken <- timestring(pre.time, Sys.time(), units="secs", show.units=FALSE)
 			if(!ask("Continue with the simulation?")){
+				
+				cat("Calculating autocorrelation and summary statistics...\n")
+				suppressWarnings(success <- try(autocorr <-  safe.autocorr.diag(normalise.mcmcfun(final.mcmc, normalise=FALSE, warn=FALSE, check.stochastic = check.stochastic)), silent=TRUE))
+				if(class(success)=="try-error"){
+					cat("An error occured while calculating the autocorrelation; aborting simulation\n")
+					return(c("Error", "An error occured while calculating the autocorrelation"))
+				}
+
+				options(show.error.messages = FALSE)
+				suppressWarnings(fsummary <- summary(combine.mcmc(final.mcmc, collapse.chains=FALSE)))
+				options(show.error.messages = TRUE)
 				pre.time <- Sys.time()-time.taken
 				cat("Simulation aborted\n\n")
 				return(c(list(pilot.mcmc=final.mcmc, end.state=pilot$end.state, req.samples=max(sample), pilot.summary=fsummary, samples.to.conv=NA, psrf=convergence, autocorr=autocorr), otheroutputs))
@@ -411,7 +455,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 		
 		
 		cat("\n")
-		additional <- run.jags(data=data, model=model, monitor=monitor,  n.chains=n.chains, inits=pilot$end.state, burnin=burnadapt, sample=moreupdates, adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE, jags = jags, psrf.target = psrf.target, normalise.mcmc=normalise.mcmc, check.stochastic = check.stochastic, modules=c(""), thin=thin, monitor.deviance=monitor.deviance, monitor.pd=monitor.pd, monitor.popt=monitor.popt, keep.jags.files=keep.jags.files, tempdir=tempdir, method=method)
+		additional <- run.jags(data=data, model=model, monitor=monitor,  n.chains=n.chains, inits=pilot$end.state, burnin=burnadapt, sample=moreupdates, adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE, jags = jags, psrf.target = psrf.target, normalise.mcmc=normalise.mcmc, check.stochastic = check.stochastic, modules=modules, factories=factories, thin=thin, monitor.deviance=monitor.deviance, monitor.pd=monitor.pd, monitor.pd.i=monitor.pd.i, monitor.popt=monitor.popt, keep.jags.files=keep.jags.files, tempdir=tempdir, method=method)
 		if(any(additional=="Unable to load coda files")){
 			cat("An error occured during the simulation\n\n")
 			return(c("Error", "An error occured during the simulation"))
@@ -427,7 +471,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 				cat("\nThe simulation crashed; retrying...",newlines,sep="")			
 				crash.retry <- crash.retry - 1
 				oldadd <- additional
-				additional <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits = pilot$end.state, burnin=burnadapt, sample=moreupdates, adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE, jags = jags, psrf.target = psrf.target, normalise.mcmc=normalise.mcmc, check.stochastic = check.stochastic, modules=c(""), thin=thin, monitor.deviance=monitor.deviance, monitor.pd=monitor.pd, monitor.popt=monitor.popt, keep.jags.files=keep.jags.files, tempdir=tempdir, method=method)
+				additional <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits = pilot$end.state, burnin=burnadapt, sample=moreupdates, adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE, jags = jags, psrf.target = psrf.target, normalise.mcmc=normalise.mcmc, check.stochastic = check.stochastic, modules=modules, factories=factories, thin=thin, monitor.deviance=monitor.deviance, monitor.pd=monitor.pd, monitor.pd.i=monitor.pd.i, monitor.popt=monitor.popt, keep.jags.files=keep.jags.files, tempdir=tempdir, method=method)
 				if(any(additional=="Unable to load coda files")){
 					cat("An error occured during the simulation\n\n")
 					return(c("Error", "An error occured during the simulation"))
@@ -449,18 +493,19 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 	
 	if(monitor.deviance) otheroutputs$deviance <- additional$deviance
 	if(monitor.pd) otheroutputs$pd <- additional$pd
+	if(monitor.pd.i) otheroutputs$pd.i <- additional$pd.i
 	if(monitor.popt) otheroutputs$popt <- additional$popt
 	
 	
 	#final.mcmc <- window(final.mcmc, start=max(burnin))
 	n.iters <- niter(final.mcmc)
 	
-	suppressWarnings(success <- try(convergence <- safe.gelman.diag(normalise.mcmc(final.mcmc, normalise=normalise.mcmc, warn=FALSE, check.stochastic = check.stochastic), warn=TRUE, transform=FALSE, autoburnin=TRUE), silent=TRUE))
+	suppressWarnings(success <- try(convergence <- safe.gelman.diag(normalise.mcmcfun(final.mcmc, normalise=normalise.mcmc, warn=FALSE, check.stochastic = check.stochastic), warn=TRUE, transform=FALSE, autoburnin=TRUE), silent=TRUE))
 	if(class(success)=="try-error"){
 		cat("An error occured while calculating the Gelman-Rubin statistic; aborting simulation.  Check that different chains have not been given the same starting values and random seeds.\n")
 		return(c("Error", "An error occured while calculating the Gelman-Rubin statistic"))
 	}
-	suppressWarnings(success <- try(autocorrelation <- autocorr.diag(normalise.mcmc(final.mcmc, normalise=FALSE, warn=FALSE, check.stochastic = check.stochastic)), silent=TRUE))
+	suppressWarnings(success <- try(autocorrelation <- safe.autocorr.diag(normalise.mcmcfun(final.mcmc, normalise=FALSE, warn=FALSE, check.stochastic = check.stochastic)), silent=TRUE))
 	if(class(success)=="try-error"){
 		cat("An error occured while calculating the autocorrelation; aborting simulation\n")
 		return(c("Error", "An error occured while calculating the autocorrelation"))
@@ -497,7 +542,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 	}
 	
 	if(unconverged > 0 & !killautocorr){
-		cat(paste("WARNING:  The Gelman-Rubin statistic for ", unconverged, " parameter(s) are above ", psrf.target, " for the final chains", mpsrfstring, ".  The chains may have fallen out of convergence.  It is important to assess convergence manually before relying on inference from these chains.\n", sep=""))
+		cat(paste("WARNING:  The Gelman-Rubin statistic for ", unconverged, " parameter", if(unconverged>1) "s", " are above ", psrf.target, " for the final chains", mpsrfstring, ".  The chains may have fallen out of convergence.  It is important to assess convergence manually before relying on inference from these chains.\n", sep=""))
 	}
 	
 	if(n.params==1) convergence$mpsrf <- NULL
@@ -516,14 +561,14 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 	if(killautocorr==FALSE){
 		if(!is.na(param.autocorr)){
 			if(autocorrelated > 0 & moreupdates > 0){
-				cat("REMINDER:  There was a high degree of autocorrelation for ", autocorrelated, " parameter(s)\n", sep="")
+				cat("REMINDER:  There was a high degree of autocorrelation for ", autocorrelated, " parameter", if(autocorrelated>1) "s", "\n", sep="")
 			}else{
 				#cat("Convergence achieved for this run\n")
 			}
 		}else{
 			cat("Autocorrelation could not be calculated for these chains\n")
 		}
-		unused <- normalise.mcmc(final.mcmc, normalise=FALSE, warn="warning", check.stochastic = check.stochastic)  # SO THAT WARNING OF NON STOCHASTICITY IS PRINTED
+		unused <- normalise.mcmcfun(final.mcmc, normalise=FALSE, warn="warning", check.stochastic = check.stochastic)  # SO THAT WARNING OF NON STOCHASTICITY IS PRINTED
 	}
 	
 	if(plots==TRUE){
